"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lua = require("../../LuaAST");
const diagnostics_1 = require("../utils/diagnostics");
const lua_ast_1 = require("../utils/lua-ast");
const lualib_1 = require("../utils/lualib");
const call_1 = require("../visitors/call");
function createStringCall(methodName, tsOriginal, ...params) {
    const stringIdentifier = lua.createIdentifier("string");
    return lua.createCallExpression(lua.createTableIndexExpression(stringIdentifier, lua.createStringLiteral(methodName)), params, tsOriginal);
}
function transformStringPrototypeCall(context, node) {
    const expression = node.expression;
    const signature = context.checker.getResolvedSignature(node);
    const params = call_1.transformArguments(context, node.arguments, signature);
    const caller = context.transformExpression(expression.expression);
    const expressionName = expression.name.text;
    switch (expressionName) {
        case "replace":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringReplace, node, caller, ...params);
        case "concat":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringConcat, node, caller, ...params);
        case "indexOf":
            const stringExpression = createStringCall("find", node, caller, params[0], params[1] ? lua_ast_1.createExpressionPlusOne(params[1]) : lua.createNilLiteral(), lua.createBooleanLiteral(true));
            return lua.createBinaryExpression(lua.createBinaryExpression(stringExpression, lua.createNumericLiteral(0), lua.SyntaxKind.OrOperator), lua.createNumericLiteral(1), lua.SyntaxKind.SubtractionOperator, node);
        case "substr":
            if (node.arguments.length === 1) {
                const argument = context.transformExpression(node.arguments[0]);
                const arg1 = lua_ast_1.createExpressionPlusOne(argument);
                return createStringCall("sub", node, caller, arg1);
            }
            else {
                const sumArg = lua.createBinaryExpression(params[0], params[1], lua.SyntaxKind.AdditionOperator);
                return createStringCall("sub", node, caller, lua_ast_1.createExpressionPlusOne(params[0]), sumArg);
            }
        case "substring":
            if (node.arguments.length === 1) {
                const arg1 = lua_ast_1.createExpressionPlusOne(params[0]);
                return createStringCall("sub", node, caller, arg1);
            }
            else {
                const arg1 = lua_ast_1.createExpressionPlusOne(params[0]);
                const arg2 = params[1];
                return createStringCall("sub", node, caller, arg1, arg2);
            }
        case "slice":
            if (node.arguments.length === 0) {
                return caller;
            }
            else if (node.arguments.length === 1) {
                const arg1 = lua_ast_1.createExpressionPlusOne(params[0]);
                return createStringCall("sub", node, caller, arg1);
            }
            else {
                const arg1 = lua_ast_1.createExpressionPlusOne(params[0]);
                const arg2 = params[1];
                return createStringCall("sub", node, caller, arg1, arg2);
            }
        case "toLowerCase":
            return createStringCall("lower", node, caller);
        case "toUpperCase":
            return createStringCall("upper", node, caller);
        case "trim":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringTrim, node, caller);
        case "trimEnd":
        case "trimRight":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringTrimEnd, node, caller);
        case "trimStart":
        case "trimLeft":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringTrimStart, node, caller);
        case "split":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringSplit, node, caller, ...params);
        case "charAt":
            const firstParamPlusOne = lua_ast_1.createExpressionPlusOne(params[0]);
            return createStringCall("sub", node, caller, firstParamPlusOne, firstParamPlusOne);
        case "charCodeAt": {
            const firstParamPlusOne = lua_ast_1.createExpressionPlusOne(params[0]);
            return createStringCall("byte", node, caller, firstParamPlusOne);
        }
        case "startsWith":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringStartsWith, node, caller, ...params);
        case "endsWith":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringEndsWith, node, caller, ...params);
        case "repeat":
            const math = lua.createIdentifier("math");
            const floor = lua.createStringLiteral("floor");
            const parameter = lua.createCallExpression(lua.createTableIndexExpression(math, floor), [params[0]]);
            return createStringCall("rep", node, caller, parameter);
        case "padStart":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringPadStart, node, caller, ...params);
        case "padEnd":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.StringPadEnd, node, caller, ...params);
        default:
            context.diagnostics.push(diagnostics_1.unsupportedProperty(expression.name, "string", expressionName));
    }
}
exports.transformStringPrototypeCall = transformStringPrototypeCall;
function transformStringConstructorCall(context, node) {
    const expression = node.expression;
    const signature = context.checker.getResolvedSignature(node);
    const params = call_1.transformArguments(context, node.arguments, signature);
    const expressionName = expression.name.text;
    switch (expressionName) {
        case "fromCharCode":
            return lua.createCallExpression(lua.createTableIndexExpression(lua.createIdentifier("string"), lua.createStringLiteral("char")), params, node);
        default:
            context.diagnostics.push(diagnostics_1.unsupportedProperty(expression.name, "String", expressionName));
    }
}
exports.transformStringConstructorCall = transformStringConstructorCall;
function transformStringProperty(context, node) {
    switch (node.name.text) {
        case "length":
            const expression = context.transformExpression(node.expression);
            return lua.createUnaryExpression(expression, lua.SyntaxKind.LengthOperator, node);
        default:
            context.diagnostics.push(diagnostics_1.unsupportedProperty(node.name, "string", node.name.text));
    }
}
exports.transformStringProperty = transformStringProperty;
//# sourceMappingURL=string.js.map